/*
 * Segment 2 (KEx),  SWE 2.DE Statischer Import MIF
 * Copyright (C) 2007 BitCtrl Systems GmbH 
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */


package de.bsvrz.kex.mif.kb;

import java.util.ArrayList;
import java.util.List;

import com.vividsolutions.jts.geom.Coordinate;
import com.vividsolutions.jts.geom.Geometry;

import de.bsvrz.dav.daf.main.ClientDavConnection;
import de.bsvrz.dav.daf.main.config.SystemObjectType;
import de.bsvrz.kex.mif.konfigdaten.ImportException;
import de.bsvrz.sys.funclib.bitctrl.geolib.WGS84Punkt;
import de.bsvrz.sys.funclib.kbgen.ProxyConfigurationObject;

/**
 * Konfigurationsobjekt.
 *
 * @author BitCtrl Systems GmbH, Gieseler
 */
public interface KonfigurationsObjekt {

	/**
	 * F&uuml;gt Koordinaten hinzu.
	 *
	 * @param punkte
	 *            Punktkoordinaten
	 */
	void addKoordinaten(ArrayList<WGS84Punkt> punkte);

	/**
	 * Gibt den Zieltyp des Objektes im DAV als String zur&uuml;ck.
	 *
	 * @return Zieltyp
	 */
	String getDavobjektyp();

	/**
	 * Gibt den Zieltyp des Objektes im DAV zur&uuml;ck.
	 *
	 * @return {@link SystemObjectType}
	 */
	SystemObjectType getDavTyp();

	/**
	 * Gibt die Geometrie des Objektes im DAVzur&uuml;ck.
	 *
	 * @return {@link SystemObjectType}
	 */
	Geometry getGeometrie();

	/**
	 * Gibt die Koordinaten des Objektes zur&uuml;ck.
	 *
	 * @return Koordinatenliste
	 */
	ArrayList<WGS84Punkt> getKoordinaten();

	/**
	 * Gibt die L&auml;ge des Polygonzuges des Objektes zur&uuml;ck.
	 *
	 * @return L&auml;nge
	 */
	double getLaenge();

	/**
	 * Gibt den Namen zur&uuml;ck.
	 *
	 * @return Name
	 */
	String getName();

	/**
	 * Gibt die PID zur&uuml;ck.
	 *
	 * @return Pid
	 */
	String getPid();

	/**
	 * Gibt das {@link ProxyConfigurationObject} zum Objekt zur&uuml;ck.
	 *
	 * @param dav
	 *            {@link ClientDavConnection}
	 * @return {@link ProxyConfigurationObject}
	 * @throws ImportException
	 *             bei Ausnahmen
	 */
	List<ProxyConfigurationObject> getProxyObjekt(ClientDavConnection dav) throws ImportException;

	/**
	 * Setzt das Flag, ob das Objekt importiert werden soll.
	 *
	 * @param importieren
	 *            true/false
	 */
	void setImportieren(boolean importieren);

	/**
	 * Setzt die Koordinaten.
	 *
	 * @param punkte
	 *            Punktkoordinaten
	 *
	 */
	void setKoordinaten(ArrayList<WGS84Punkt> punkte);

	/**
	 * Setzt die Koordinaten.
	 *
	 * @param featureKoordinatenWGS84
	 *            Feature-Koordinate
	 */
	void setKoordinaten(Coordinate[] featureKoordinatenWGS84);

	/**
	 * Setzt die L&auml;ge des Polygonzuges des Objektes.
	 *
	 * @param laenge
	 *            L&auml;nge
	 */
	void setLaenge(double laenge);

	/**
	 * Setzt den Namen.
	 *
	 * @param name
	 *            Name
	 */
	void setName(String name);

	/**
	 * Setzt die PID.
	 *
	 * @param pid
	 *            PID
	 *
	 */
	void setPid(String pid);

	/**
	 * Setzt den DAV-Zieltyp des Objektes.
	 *
	 * @param zieltyp
	 *            Zieltyp als String
	 */
	void setZieltyp(String zieltyp);

	/**
	 * Test, ob das Objekt importiert werden soll.
	 *
	 * @return true, wenn importiert werden soll, sonst false
	 */
	boolean sollImportieren();
}
