/*
 * Segment 2 (KEx),  SWE 2.DE Statischer Import MIF
 * Copyright (C) 2007 BitCtrl Systems GmbH
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.kex.mif.vew;

import java.awt.Color;
import java.awt.Component;
import java.awt.Container;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.ListSelectionModel;
import javax.swing.ScrollPaneConstants;
import javax.swing.SwingUtilities;
import javax.swing.border.LineBorder;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.filechooser.FileFilter;

import de.bsvrz.kex.mif.konfigdaten.ImportException;
import de.bsvrz.kex.mif.konfigdaten.KonfigurationsDaten;
import de.bsvrz.kex.mif.mif.MifDaten;
import de.bsvrz.kex.mif.mif.MifTyp;
// import de.bsvrz.pat.sysbed.preselection.lists.PreselectionListsListener;

/**
 * Hauptklasse f&uuml;r die SWE 2.MIF mit GUI.
 *
 * @author BitCtrl Systems GmbH, Gieseler
 */
public class KexMIFgui extends KexMIF implements ActionListener, DocumentListener, ImportListener, ItemListener {
	/**
	 * Dateifilter f&uuml;r MIF-Dateien.
	 *
	 * @author BitCtrl Systems GmbH, Gieseler
	 */
	public class MifFileFilter extends FileFilter {

		@Override
		public boolean accept(final File f) {
			if (f.isDirectory()) {
				return true;
			}

			final String fname = f.getAbsolutePath();
			final int ext = fname.lastIndexOf(".");

			if (ext > 0) {
				final String theExt = fname.substring(ext + 1).toLowerCase();

				if (theExt.equals("mif")) {
					return true;
				}
			}

			return false;
		}

		@Override
		public String getDescription() {
			return "Mapinfo Dateien";
		}
	}

	/**
	 * serialVersionUID.
	 */
	private static final long serialVersionUID = 2113316346767696073L;

	/**
	 * F&uuml;gt eine Komponente zum Container hinzu.
	 *
	 * @param cont
	 * @param gbl
	 * @param c
	 * @param x
	 * @param y
	 * @param width
	 * @param height
	 * @param weightx
	 * @param weighty
	 * @param fill
	 */
	private static void addComponent(final Container cont, final GridBagLayout gbl, final Component c, final int x,
			final int y, final int width, final int height, final double weightx, final double weighty,
			final int fill) {
		final GridBagConstraints gbc = new GridBagConstraints();
		gbc.insets = new Insets(5, 5, 5, 5);
		gbc.fill = fill;
		gbc.gridx = x;
		gbc.gridy = y;
		gbc.gridwidth = width;
		gbc.gridheight = height;
		gbc.weightx = weightx;
		gbc.weighty = weighty;
		gbl.setConstraints(c, gbc);
		cont.add(c);
	}

	private GridBagLayout _gridbaglayout;
	private JButton _button_import;
	private JPanel _panel;
	private JTextField _mifdb_textfeld;
	SelectionListe _miftypen_liste;
	SelectionListe _mifspalten_liste;
	private JLabel _hl = null;
	private JTextField _kb_pfad;
	private JButton _mifdb_button;
	private JButton _kb_bt;
	// JComboBox _src_obj = null, _dest_obj = null;
	private JTextField _kb_kv_text;
	private JTextField _kb_pid_text;
	private JTextField _kb_name_text;
	private JTextArea _kb_info_text;
	private JScrollPane _kb_info_scrollPane;
	JFrame _frame;
	KexMIF _parent;
	JProgressBar progressbar;

	JDialog waitdialog;

	JCheckBox _importAll;

	private JFrame _logFrame;

	int fontsize_i = 14;

	// JComboBox _zieltyp_box;
	SelectionListe _zieltyp_box;
	private JTextField _kb_appr;

	MifTyp[] _mifTypen_db = null;

	MifTyp[] _mifTypen_import = null;

	private boolean _wait_init = false;

	String _zielTyp;

	public KexMIFgui() {
		super();
	}

	@Override
	public void actionPerformed(final ActionEvent e) {

		String property = ""; //$NON-NLS-1$

		if (e.getSource() == this._button_import) {
			try {
				MIFProperties.getInstance().setProperty(MIFProperties.PROP_KEY_KBINFO, _kb_info_text.getText());
				MIFProperties.getInstance().setProperty(MIFProperties.PROP_KEY_KV, _kb_kv_text.getText());
				MIFProperties.getInstance().setProperty(MIFProperties.PROP_KEY_KBPID, _kb_pid_text.getText());
				MIFProperties.getInstance().setProperty(MIFProperties.PROP_KEY_KBNAME, _kb_name_text.getText());
				MIFProperties.getInstance().setProperty(MIFProperties.PROP_KEY_KBPFAD, _kb_pfad.getText());

				checkParameter();

				konfigurationsdaten = new KonfigurationsDaten();
				// bilde den Dateinamen den Konfigurationsbereiches aus
				// Pfad+Pid+".xml"
				final File kbfile = new File(MIFProperties.getInstance().getProperty(MIFProperties.PROP_KEY_KBPFAD),
						MIFProperties.getInstance().getProperty(MIFProperties.PROP_KEY_KBPID) + ".xml"); //$NON-NLS-1$
				konfigurationsdaten.initialisiere(dav, kbfile.toString(),
						MIFProperties.getInstance().getProperty(MIFProperties.PROP_KEY_KV),
						MIFProperties.getInstance().getProperty(MIFProperties.PROP_KEY_KBPID),
						MIFProperties.getInstance().getProperty(MIFProperties.PROP_KEY_KBNAME),
						MIFProperties.getInstance().getProperty(MIFProperties.PROP_KEY_KBINFO));
				konfigurationsdaten
				.importiereMIF(MIFProperties.getInstance().getProperty(MIFProperties.PROP_KEY_QUELLDATEI));
				konfigurationsdaten.konvertiere(
						MIFProperties.getInstance().getProperty(MIFProperties.PROP_KEY_PIDSPALTE),
						_mifTypen_import != null ? Arrays.asList(_mifTypen_import) : null, _zielTyp);

				final JFrame editor = new ObjektEditor(konfigurationsdaten);

			} catch (final ImportException ie) {
				JOptionPane.showMessageDialog(this._frame, ie.getMessage(), "Fehler", JOptionPane.ERROR_MESSAGE); //$NON-NLS-1$
			} catch (final Exception ex) {
				ex.printStackTrace();
				JOptionPane.showMessageDialog(this._frame, ex.getMessage(), "Fehler", JOptionPane.ERROR_MESSAGE); //$NON-NLS-1$
			}
			return;
		}

		// Texteingaben
		if (e.getSource() == this._mifdb_textfeld) {
			MIFProperties.getInstance().setProperty(MIFProperties.PROP_KEY_QUELLDATEI, _mifdb_textfeld.getText());
			return;
		} else if (e.getSource() == this._kb_kv_text) {
			MIFProperties.getInstance().setProperty(MIFProperties.PROP_KEY_KV, _kb_kv_text.getText());
			return;
		} else if (e.getSource() == this._kb_pid_text) {
			MIFProperties.getInstance().setProperty(MIFProperties.PROP_KEY_KBPID, _kb_pid_text.getText());
			return;
		} else if (e.getSource() == this._kb_name_text) {
			MIFProperties.getInstance().setProperty(MIFProperties.PROP_KEY_KBNAME, _kb_name_text.getText());
			return;
		} else if (e.getSource() == this._kb_info_text) {
			MIFProperties.getInstance().setProperty(MIFProperties.PROP_KEY_KBINFO, _kb_info_text.getText());
			return;
		}

		// Buttons
		int mode = JFileChooser.FILES_ONLY;
		JTextField dest_field = null;
		FileFilter filter = null;

		// Handle open button action.
		if (e.getSource() == this._mifdb_button) {
			property = MIFProperties.PROP_KEY_QUELLDATEI;
			dest_field = this._mifdb_textfeld;

			// filter = new FileNameExtensionFilter(
			// "Mapinfo Dateien", "mif"); //$NON-NLS-1$ //$NON-NLS-2$
			filter = new MifFileFilter();
		} else if (e.getSource() == _kb_bt) {
			mode = JFileChooser.DIRECTORIES_ONLY;
			property = MIFProperties.PROP_KEY_KBPFAD;
			dest_field = this._kb_pfad;
		} else {
			return;
		}

		String filename = MIFProperties.getInstance().getProperty(property);
		if (filename == null) {
			filename = ""; //$NON-NLS-1$
		}

		final JFileChooser fc = new JFileChooser();

		fc.setSelectedFile(new File(filename));
		fc.setFileSelectionMode(mode);

		// multi-selection
		// fc.setMultiSelectionEnabled(true);

		if (filter != null) {
			fc.setFileFilter(filter);
		}

		final int returnVal = fc.showOpenDialog(KexMIFgui.this._frame);

		if (returnVal == JFileChooser.APPROVE_OPTION) {
			final String file = fc.getSelectedFile().toString();
			// fc.getSelectedFiles()
			MIFProperties.getInstance().setProperty(property, file);
			if (dest_field != null) {
				dest_field.setText(file);
			}
			if (dest_field == this._mifdb_textfeld) {
				updateMifDb();
			}
		} else {
			// log.append("Open command cancelled by user." + newline);
		}
		// log.setCaretPosition(log.getDocument().getLength());

	}

	void centerFrame() {
		_frame.pack();
		final int width = Toolkit.getDefaultToolkit().getScreenSize().width;
		final int heigth = Toolkit.getDefaultToolkit().getScreenSize().height;

		_frame.setLocation((width - _frame.getSize().width) / 2, (heigth - this._frame.getSize().height) / 2);
	}

	@Override
	public void changedUpdate(final DocumentEvent arg0) {
		// TODO Automatisch erstellter Methoden-Stub

	}

	void enableComponent(final Container cont, final boolean enable) {
		for (final Component c : cont.getComponents()) {
			c.setEnabled(enable);
			if (c instanceof Container) {
				enableComponent((Container) c, enable);
			}
		}
	}

	@Override
	public void insertUpdate(final DocumentEvent arg0) {
		// TODO Automatisch erstellter Methoden-Stub

	}

	/*
	 * (non-Javadoc)
	 *
	 * @see
	 * java.awt.event.ItemListener#itemStateChanged(java.awt.event.ItemEvent)
	 */
	@Override
	public void itemStateChanged(final ItemEvent e) {
		final Object source = e.getItemSelectable();

		if (source == _importAll) {
			if (e.getStateChange() == ItemEvent.DESELECTED) {
				MIFProperties.getInstance().setProperty(MIFProperties.PROP_KEY_IMPORTALL, "Nein");
			}

			if (e.getStateChange() == ItemEvent.SELECTED) {
				MIFProperties.getInstance().setProperty(MIFProperties.PROP_KEY_IMPORTALL, "Ja");
			}
		}
	}

	@Override
	public void listSelectionChanged(final SelectionListe list) {
		final List<Object> objects = list.getSelectedObjects();
		if (list == _mifspalten_liste) {
			MIFProperties.getInstance().setProperty(MIFProperties.PROP_KEY_PIDSPALTE, objects.get(0).toString());
		}

		if (list == _miftypen_liste) {
			final int indizes[] = list._list.getSelectedIndices();
			_mifTypen_import = new MifTyp[indizes.length];

			for (int i = 0; i < indizes.length; i++) {
				_mifTypen_import[i] = _mifTypen_db[indizes[i]];
			}

			updateZielTypen();
		}

		if (list == _zieltyp_box) {
			final List<Object> objs = _zieltyp_box.getSelectedObjects();
			if(!objs.isEmpty()){
				_zielTyp = objs.get(0).toString();
			}else{
				_zielTyp ="";
			}
		}

	}

	@Override
	public void removeUpdate(final DocumentEvent arg0) {
		// TODO Automatisch erstellter Methoden-Stub

	}

	// /**
	// * Returns the name of a .mif file without extension
	// *
	// * @param fName The file name, possibly with .mif extension
	// *
	// * @return The name with no extension
	// *
	// * @throws FileNotFoundException if extension was other than "mif"
	// */
	// protected static String getMifName(String fName)
	// throws FileNotFoundException {
	// int ext = fName.lastIndexOf(".");
	//
	// if (ext > 0) {
	// String theExt = fName.substring(ext + 1).toLowerCase();
	//
	// if (!(theExt.equals("mif"))) {
	// throw new FileNotFoundException(
	// "Please specify a .mif file extension.");
	// }
	//
	// fName = fName.substring(0, ext);
	// }
	//
	// return fName;
	// }

	@Override
	public void startImport() throws Exception {
		// JFrame.setDefaultLookAndFeelDecorated(true);
		// _parent = parent;

		// UIDefaults uiDefaults = UIManager.getDefaults();
		//
		// uiDefaults.put( "Label.font",
		// ((Font)uiDefaults.get("Label.font")).deriveFont(18f) );
		//
		// uiDefaults.put( "TextField.font",
		// ((Font)uiDefaults.get("TextField.font")).deriveFont(18f) );
		//
		// uiDefaults.put( "Button.font",
		// ((Font)uiDefaults.get("Button.font")).deriveFont(18f) );
		//
		// uiDefaults.put( "ComboBox.font",
		// ((Font)uiDefaults.get("ComboBox.font")).deriveFont(18f) );

		this._gridbaglayout = new GridBagLayout();

		this._frame = new JFrame("MIF-Datenbasis Import"); //$NON-NLS-1$
		this._frame.setLayout(this._gridbaglayout);
		this._frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		this._frame.setSize(800, 650);

		// TEST
		// this._frame.setResizable( false );
		// this._frame.s .setDynamicLayout(false);

		this._frame.addWindowListener(new WindowAdapter() {
			@Override
			public void windowClosing(final WindowEvent e) {
				super.windowClosing(e);
				dav.disconnect(false, ""); //$NON-NLS-1$
			}
		});

		this._hl = new JLabel("Importparameter"); //$NON-NLS-1$
		this._hl.setFont(new Font(null, 0, 30));

		this._mifdb_textfeld = new JTextField(
				MIFProperties.getInstance().getProperty(MIFProperties.PROP_KEY_QUELLDATEI));
		this._mifdb_textfeld.addActionListener(this);
		this._mifdb_button = new JButton("Auswählen"); //$NON-NLS-1$
		this._mifdb_button.addActionListener(this);

		_mifspalten_liste = new SelectionListe(ListSelectionModel.SINGLE_SELECTION);
		_mifspalten_liste.addListener(this);
		_miftypen_liste = new SelectionListe(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
		_miftypen_liste.addListener(this);

		// Auswahlbox Zieltypen
		konfigurationsdaten = new KonfigurationsDaten();
		// bilde den Dateinamen den Konfigurationsbereiches aus Pfad+Pid+".xml"
		final File kbfile = new File(MIFProperties.getInstance().getProperty(MIFProperties.PROP_KEY_KBPFAD),
				MIFProperties.getInstance().getProperty(MIFProperties.PROP_KEY_KBPID) + ".xml"); //$NON-NLS-1$
		konfigurationsdaten.initialisiere(dav, kbfile.toString(),
				MIFProperties.getInstance().getProperty(MIFProperties.PROP_KEY_KV),
				MIFProperties.getInstance().getProperty(MIFProperties.PROP_KEY_KBPID),
				MIFProperties.getInstance().getProperty(MIFProperties.PROP_KEY_KBNAME),
				MIFProperties.getInstance().getProperty(MIFProperties.PROP_KEY_KBINFO));
		// _zieltyp_box = new JComboBox();
		// _zieltyp_box.setFont(_zieltyp_box.getFont().deriveFont(Font.PLAIN));
		// _zieltyp_box.setBackground(Color.white);
		// if(MIFProperties.getInstance().getProperty(MIFProperties.PROP_KEY_ZIELTYP)
		// != null)
		// _zieltyp_box.addItem(MIFProperties.getInstance().getProperty(MIFProperties.PROP_KEY_ZIELTYP));
		// _zieltyp_box.addActionListener(this)

		_zieltyp_box = new SelectionListe(ListSelectionModel.SINGLE_SELECTION);
		_zieltyp_box.addListener(this);
		_zielTyp = MIFProperties.getInstance().getProperty(MIFProperties.PROP_KEY_ZIELTYP);

		// _zieltyp_box.addItem("ABC");
		// Dummy ????
		// _konfigurationsdaten.getMoeglicheZieltypen(null);

		// _dest_obj.setFont(this._dest_obj.getFont().deriveFont(Font.PLAIN));
		// this._abweichungen_jn.setFont(new Font(null, Font.PLAIN, 18 ));
		;

		this._kb_pfad = new JTextField(MIFProperties.getInstance().getProperty(MIFProperties.PROP_KEY_KBPFAD));
		this._kb_pfad.addActionListener(this);
		this._kb_bt = new JButton("Auswählen"); //$NON-NLS-1$
		this._kb_bt.addActionListener(this);

		this._kb_kv_text = new JTextField(MIFProperties.getInstance().getProperty(MIFProperties.PROP_KEY_KV));
		this._kb_kv_text.addActionListener(this);

		this._kb_pid_text = new JTextField(MIFProperties.getInstance().getProperty(MIFProperties.PROP_KEY_KBPID));
		this._kb_pid_text.addActionListener(this);
		this._kb_name_text = new JTextField(MIFProperties.getInstance().getProperty(MIFProperties.PROP_KEY_KBNAME));
		this._kb_name_text.addActionListener(this);

		this._kb_info_text = new JTextArea(MIFProperties.getInstance().getProperty(MIFProperties.PROP_KEY_KBINFO), 5,
				40);
		this._kb_info_text.getDocument().addDocumentListener(this);
		this._kb_info_text.setLineWrap(true);
		this._kb_info_text.setWrapStyleWord(true);
		// this._kb_info_text.setFont(new Font(null, 0, 18 ));
		this._kb_info_text.setBorder(new LineBorder(Color.GRAY));
		_kb_info_text.setRows(4);
		_kb_info_text.setMinimumSize(new Dimension(100, 60));
		_kb_info_scrollPane = new JScrollPane(_kb_info_text, ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED,
				ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
		// _kb_info_scrollPane.setPreferredSize(new Dimension(200, 80));
		// _kb_info_scrollPane.setMinimumSize(new Dimension(200, 100));

		// t.setFont( new Font(Font.SANS_SERIF, style, size) );
		// this._kb_info_p = new JScrollPane(this._kb_info_text );

		this._kb_appr = new JTextField(MIFProperties.getInstance().getProperty(MIFProperties.PROP_KEY_APPROXIMATION));
		this._kb_appr.addActionListener(this);

		this._button_import = new JButton("Import starten"); //$NON-NLS-1$
		this._button_import.setEnabled(true);
		this._button_import.addActionListener(this);

		_importAll = new JCheckBox("");
		// _importAll.setMnemonic(KeyEvent.VK_C);
		_importAll.setSelected(MIFProperties.getInstance().istAktiv(MIFProperties.PROP_KEY_IMPORTALL));
		_importAll.addItemListener(this);

		_panel = new JPanel(new FlowLayout(FlowLayout.CENTER));
		_panel.add(this._hl);

		// x y w h wx wy
		KexMIFgui.addComponent(this._frame, this._gridbaglayout, this._hl, 0, 0, 4, 1, 0, 0, GridBagConstraints.NONE);

		KexMIFgui.addComponent(this._frame, this._gridbaglayout, new JLabel("MIF-Datenbasis:"), 0, 1, 1, 1, 0, 0,
				GridBagConstraints.HORIZONTAL);
		KexMIFgui.addComponent(this._frame, this._gridbaglayout, this._mifdb_textfeld, 1, 1, 2, 1, 2, 0,
				GridBagConstraints.HORIZONTAL);
		KexMIFgui.addComponent(this._frame, this._gridbaglayout, this._mifdb_button, 3, 1, 1, 1, 0, 0,
				GridBagConstraints.HORIZONTAL);

		KexMIFgui.addComponent(this._frame, this._gridbaglayout, new JLabel("MIF-Spalte für PID:"), 0, 2, 1, 1, 0, 0,
				GridBagConstraints.HORIZONTAL);
		KexMIFgui.addComponent(this._frame, this._gridbaglayout, new JLabel("MIF-Quellobjekttyp:"), 1, 2, 1, 1, 0, 0,
				GridBagConstraints.HORIZONTAL);
		KexMIFgui.addComponent(this._frame, this._gridbaglayout, new JLabel("Zielobjekttyp:"), 2, 2, 1, 1, 0, 0,
				GridBagConstraints.HORIZONTAL);

		// JPanel _panelt = new JPanel(new FlowLayout(FlowLayout.LEFT));
		// _panelt.add(new JLabel("Zielobjekttyp: ("));
		// JCheckBox _importAll = new JCheckBox("importiere nicht konvertierbare
		// auf Basistyp");
		// //_importAll.setMnemonic(KeyEvent.VK_C);
		// _importAll.setSelected(true);
		// _panelt.add( _importAll);
		// _panelt.add(new JLabel(")"));
		// addComponent(this._frame, this._gridbaglayout, _panelt,
		// 2, 2, 2, 1, 0, 0, GridBagConstraints.HORIZONTAL);

		final JPanel _panelt = new JPanel();
		_panelt.setLayout(new BoxLayout(_panelt, BoxLayout.Y_AXIS));
		// _panelt.add(new JLabel("Zielobjekttyp: ("));

		_panelt.add(_importAll);
		_panelt.add(new JLabel("importiere nicht"));
		_panelt.add(new JLabel("konvertierbare"));
		_panelt.add(new JLabel("Quellobjekttypen"));
		_panelt.add(new JLabel("auf DAV-Basistyp"));
		KexMIFgui.addComponent(this._frame, this._gridbaglayout, _panelt, 3, 3, 1, 1, 0, 0,
				GridBagConstraints.VERTICAL);

		KexMIFgui.addComponent(this._frame, this._gridbaglayout, _mifspalten_liste, 0, 3, 1, 2, 0, 0,
				GridBagConstraints.HORIZONTAL);
		KexMIFgui.addComponent(this._frame, this._gridbaglayout, _miftypen_liste, 1, 3, 1, 2, 0, 0,
				GridBagConstraints.HORIZONTAL);

		KexMIFgui.addComponent(this._frame, this._gridbaglayout, _zieltyp_box, 2, 3, 1, 2, 0, 0,
				GridBagConstraints.HORIZONTAL);

		// addComponent(this._frame, this._gridbaglayout, new
		// JLabel("Zielobjekttyp:"),
		// 0, 4, 1, 1, 0, 0, GridBagConstraints.HORIZONTAL);
		// addComponent(this._frame, this._gridbaglayout, this._zieltyp_box,
		// 1, 4, 1, 1, 0, 0, GridBagConstraints.HORIZONTAL);

		KexMIFgui.addComponent(this._frame, this._gridbaglayout, new JLabel("Pfad Konfigurationsbereich:"), //$NON-NLS-1$
				0, 5, 1, 1, 0, 0, GridBagConstraints.HORIZONTAL);
		KexMIFgui.addComponent(this._frame, this._gridbaglayout, this._kb_pfad, 1, 5, 2, 1, 2, 0,
				GridBagConstraints.HORIZONTAL);
		KexMIFgui.addComponent(this._frame, this._gridbaglayout, this._kb_bt, 3, 5, 1, 1, 0, 0,
				GridBagConstraints.HORIZONTAL);

		KexMIFgui.addComponent(this._frame, this._gridbaglayout, new JLabel("Konfigurationsverantwortlicher:"), //$NON-NLS-1$
				0, 6, 1, 1, 0, 0, GridBagConstraints.HORIZONTAL);
		KexMIFgui.addComponent(this._frame, this._gridbaglayout, this._kb_kv_text, 1, 6, 2, 1, 2, 0,
				GridBagConstraints.HORIZONTAL);

		KexMIFgui.addComponent(this._frame, this._gridbaglayout, new JLabel("Konfigurationsbereich PID:"), //$NON-NLS-1$
				0, 7, 1, 1, 0, 0, GridBagConstraints.HORIZONTAL);
		KexMIFgui.addComponent(this._frame, this._gridbaglayout, this._kb_pid_text, 1, 7, 2, 1, 2, 0,
				GridBagConstraints.HORIZONTAL);

		KexMIFgui.addComponent(this._frame, this._gridbaglayout, new JLabel("Konfigurationsbereich Name:"), //$NON-NLS-1$
				0, 8, 1, 1, 0, 0, GridBagConstraints.HORIZONTAL);
		KexMIFgui.addComponent(this._frame, this._gridbaglayout, this._kb_name_text, 1, 8, 2, 1, 0, 0,
				GridBagConstraints.HORIZONTAL);

		KexMIFgui.addComponent(this._frame, this._gridbaglayout, new JLabel("Konfigurationsbereich Info:"), //$NON-NLS-1$
				0, 9, 1, 3, 0, 0, GridBagConstraints.HORIZONTAL);
		KexMIFgui.addComponent(this._frame, this._gridbaglayout, _kb_info_scrollPane, 1, 9, 2, 3, 0, 0,
				GridBagConstraints.HORIZONTAL);

		KexMIFgui.addComponent(this._frame, this._gridbaglayout, new JLabel("Approximation:"), 0, 12, 1, 1, 0, 0,
				GridBagConstraints.HORIZONTAL);
		KexMIFgui.addComponent(this._frame, this._gridbaglayout, this._kb_appr, 1, 12, 1, 1, 0, 0,
				GridBagConstraints.HORIZONTAL);
		KexMIFgui.addComponent(this._frame, this._gridbaglayout, new JLabel("Grad"), 2, 12, 1, 1, 0, 0,
				GridBagConstraints.HORIZONTAL);

		KexMIFgui.addComponent(this._frame, this._gridbaglayout, this._button_import, 0, 13, 4, 2, 0, 1.0,
				GridBagConstraints.NONE);

		// Abstand nach Button
		KexMIFgui.addComponent(this._frame, this._gridbaglayout, new JLabel(), 0, 15, 4, 1, 0, 1.0,
				GridBagConstraints.HORIZONTAL);

		if ((MIFProperties.getInstance().getProperty(MIFProperties.PROP_KEY_QUELLDATEI) != null)
				&& (MIFProperties.getInstance().getProperty(MIFProperties.PROP_KEY_QUELLDATEI).length() != 0)) {
			updateMifDb();
			_wait_init = true;
		} else {
			centerFrame();
			this._frame.setVisible(true);
		}
	}

	private void updateMifDb() {
		SwingUtilities.invokeLater(new Runnable() {
			@Override
			public void run() {
				_frame.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
				final String dbfile = MIFProperties.getInstance().getProperty(MIFProperties.PROP_KEY_QUELLDATEI);

				final MifDaten md = new MifDaten();
				try {
					md.importiere(dbfile);

					final List<String> miftypenstrings = new ArrayList<String>();

					_mifTypen_db = md.getMifTypen();

					for (int i = 0; i < md.getMifTypen().length; i++) {
						miftypenstrings.add((md.getMifTypen()[i]).toString());
					}

					_miftypen_liste.setObjects(miftypenstrings);

					_mifspalten_liste.setObjects(Arrays.asList(md.getSpaltenNamen()));

				} catch (final ImportException e) {
					JOptionPane.showMessageDialog(_frame, e.getMessage(), "Fehler", JOptionPane.ERROR_MESSAGE); //$NON-NLS-1$
					_frame.setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
				} finally {
					_frame.setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));

					if (_wait_init) {
						centerFrame();
						_frame.setVisible(true);
						_wait_init = false;
					}
				}
			}
		});
	}

	private void updateZielTypen() {
		SwingUtilities.invokeLater(new Runnable() {
			@Override
			public void run() {
				// _frame.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
				//
				// _zieltyp_box.removeAllItems();
				final List<String> zieltypen = konfigurationsdaten.getMoeglicheZieltypen(_mifTypen_import);
				// for(String t : zieltypen)
				// _zieltyp_box.addItem(t);
				_zieltyp_box.setObjects(zieltypen);
				_zielTyp = MIFProperties.getInstance().getProperty(MIFProperties.PROP_KEY_ZIELTYP);
				//
				// _frame.setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
				// //_frame.pack();
				// }
			}
		});
	}

}
